﻿/*
 * 
 *          DremDOS
 *          Main Kernel Class (Kernel.cs)
 * 
 *          Contains the BUI (Border User Interface) and main controlling logic for Terminals.
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using Sys = Cosmos.System;
using System;
using System.Collections.Generic;
using System.Text;
using Cosmos.HAL.Drivers.PCI.Video;
using Cosmos.System.Graphics;
using System.Runtime.InteropServices;
using System.Diagnostics;
using Cosmos.System;
using System.Drawing;
using Cosmos.Core.IOGroup;
using Console = System.Console;
using DremDOS.Core.Terminal;
using DremDOS.Core.Graphics;
using DremDOS.Core.Drivers;
using DremDOS.Core.Sound;

namespace DremDOS
{
    public class Kernel : Sys.Kernel
    {
        public static int _SCREEN_WIDTH;
        public static int _SCREEN_HEIGHT;
        public static string _OS_NAME = "DremDOS";
        public static int _OS_VERSION_MAJOR = 0;
        public static int _OS_VERSION_MINOR = 1;
        public static int _OS_VERSION_PATCH = 0;
        public static string _OS_VERSION_TACKON = " (Multitasking Console Test)";
        public static string _OS_VERSION_FULL = _OS_VERSION_MAJOR + "." + _OS_VERSION_MINOR + "." + _OS_VERSION_PATCH + _OS_VERSION_TACKON;
        public static int CurrentConsole = 1;
        public static int TotalConsoles = 1;
        public static Color BorderColor = Color.Purple;
        public static Pen BorderPen = new Pen(Color.Purple);
        public static Pen WhitePen = new Pen(Color.White);

        private static Canvas canvas;
        private static Terminal MainTerminal;
        //private static VGATerminal ConsoleA = new VGATerminal(MainTerminal, 80, 25, 50, 50);
        private static VGATerminal ConsoleA;
        public static MouseDriver mouseDriver;
        public static Sys.Graphics.Fonts.PCScreenFont Font;
        //VMWareSVGAII driver;

        private bool ConsoleMode;
        private bool BlackBoxHasBeenFixed;
        protected override void BeforeRun()
        {
            ConsoleMode = false;
            Console.Clear();
            Sound.Startup();
            Console.ForegroundColor = ConsoleColor.Blue;
            Console.WriteLine(@"                                           _..._");
            Console.ForegroundColor = ConsoleColor.Red;
            Console.Write(@"    ___                ___  ____  ____   ");
            Console.ForegroundColor = ConsoleColor.DarkBlue;
            Console.WriteLine(@".:     '.");
            Sound.Startup();
            Console.ForegroundColor = ConsoleColor.Yellow;
            Console.Write(@"   / _ \_______ __ _  / _ \/ __ \/ __/  ");
            Console.ForegroundColor = ConsoleColor.DarkBlue;
            Console.WriteLine(@":::::     :");
            Sound.Startup();
            Console.ForegroundColor = ConsoleColor.Green;
            Console.Write(@"  / // / __/ -_)  ' \/ // / /_/ /\ \    ");
            Console.ForegroundColor = ConsoleColor.DarkMagenta;
            Console.WriteLine(@":::::::   :");
            Sound.Startup();
            Console.ForegroundColor = ConsoleColor.Blue;
            Console.Write(@" /____/_/  \__/_/_/_/____/\____/___/    ");
            Console.ForegroundColor = ConsoleColor.DarkMagenta;
            Console.WriteLine(@"`::::::::.'");
            Console.ForegroundColor = ConsoleColor.Magenta;
            Console.WriteLine(@"                                          `':::''");
            Console.ForegroundColor = ConsoleColor.Red;
            Console.WriteLine("\nNOTICE: THIS SOFTWARE IS PROVIDED AS-IS AND INCLUDES ABSOLUTELY NO WARRENTY. ANY DAMAGE TO YOUR DEVICE AND/OR DATA IS ON YOU!");
            Console.ForegroundColor = ConsoleColor.DarkRed;
            Console.WriteLine("YOU HAVE BEEN WARNED!");
            Console.ForegroundColor = ConsoleColor.Blue;
            Console.WriteLine("\nNote: Because of a bug in the PS/2 driver, Enter is *, Backspace is /");
            //Console.WriteLine("Left, Right, Up, and Down keys are Num4, Num6, Num8, and Num2, respectively.");
            Console.ForegroundColor = ConsoleColor.White;
            Console.WriteLine("\n");
            bool ValidResolutionChoiceFlag = false;
            while (!ValidResolutionChoiceFlag) {
                Console.WriteLine("    1.) 800x600    2.) 1024x768    3.) 1280x720");
                Console.WriteLine("    4.) 1920x1080");
                Console.WriteLine("    C.) Single console mode");
                Console.WriteLine("Choose resolution:");
                string ResolutionChoice = Console.ReadKey().KeyChar.ToString();

                switch(ResolutionChoice) {
                    case "1":
                        _SCREEN_WIDTH = 800;
                        _SCREEN_HEIGHT = 600;
                        ValidResolutionChoiceFlag = true;
                        break;
                    case "2":
                        _SCREEN_WIDTH = 1024;
                        _SCREEN_HEIGHT = 768;
                        ValidResolutionChoiceFlag = true;
                        break;
                    case "3":
                        _SCREEN_WIDTH = 1280;
                        _SCREEN_HEIGHT = 720;
                        ValidResolutionChoiceFlag = true;
                        break;
                    case "4":
                        _SCREEN_WIDTH = 1920;
                        _SCREEN_HEIGHT = 1080;
                        ValidResolutionChoiceFlag = true;
                        break;
                    case "c":
                    case "C":
                        _SCREEN_WIDTH = 80;
                        _SCREEN_HEIGHT = 24;
                        ValidResolutionChoiceFlag = true;
                        ConsoleMode = true;
                        break;
                    default:
                        break;
                }
            }

            if (!ConsoleMode)
            {
                Font = Sys.Graphics.Fonts.PCScreenFont.Default;
                canvas = FullScreenCanvas.GetFullScreenCanvas(new Mode(_SCREEN_WIDTH, _SCREEN_HEIGHT, ColorDepth.ColorDepth32));
            }
        }

        protected override void Run()
        {
            try
            {
                Terminal MainTerminal = new Terminal(ConsoleMode);
                if (!ConsoleMode)
                {
                    canvas.Clear(BorderColor);
                    canvas.Display();

                    MouseDriver mouseDriver = new MouseDriver(checked((uint)_SCREEN_WIDTH), checked((uint)_SCREEN_HEIGHT));
                    //VMWareSVGAII driver = new VMWareSVGAII();
                    //VMWareSVGAII driver = new VMWareSVGAII();
                    //driver.SetMode(checked((uint)_SCREEN_WIDTH), checked((uint)_SCREEN_HEIGHT));
                    //driver.Update(0, 0, checked((uint)_SCREEN_WIDTH), checked((uint)_SCREEN_HEIGHT));

                    VGATerminal ConsoleA = new VGATerminal(MainTerminal, (_SCREEN_WIDTH / 8) - 12, (_SCREEN_HEIGHT / 16) - 6, 50, 50);
                    ConsoleA.ShowCursor = true;
                    ConsoleA.EnableWrite = true;

                    //Console.Beep(100, 100);
                    MainTerminal.SetLinkedVGATerminal(ConsoleA);
                    MainTerminal.ShowOutput = true;
                    //Console.Beep(500, 100);
                    canvas.Clear(BorderColor);
                    canvas.Display();
                    //Console.Beep(1000, 100);

                    //canvas.DrawACSIIString(WhitePen, _OS_NAME + " " + _OS_VERSION_FULL, 50, 5);
                    //canvas.DrawACSIIString(WhitePen, ConsoleA.VGATerminalName, 50, 20);
                    canvas.DrawString(_OS_NAME + " " + _OS_VERSION_FULL, Font, WhitePen, 50, 5);
                    canvas.DrawString(ConsoleA.VGATerminalName, Font, WhitePen, 50, 20);

                    //Console.Beep(1500, 1000);
                    ConsoleA.UpdateEntireScreen(true);
                    //Console.Beep(2000, 1000);
                    int TempX;
                    int TempY;
                    BUI.DrawBUI(canvas);
                    canvas.Display();
                    BlackBoxHasBeenFixed = false;
                }

                if (!ConsoleMode)
                {
                    //ConsoleA.RunTerminalCommand("command");
                }
                else
                {
                    Console.Clear();
                    MainTerminal.RunCommand("command");
                }
                ConsoleA.WriteText("Hello World!", 0, 0);
                Console.Beep(2500, 1000);
                while (true)
                {
                    //Console.Beep(3000, 1000);
                    //ConsoleKeyInfo inputchar = Console.ReadKey();
                    //ConsoleA.InterpretKeyInput(inputchar);
                    //ConsoleA.UpdateEntireScreen();
                    //int inputchar = Console.Read();
                    int InputCharInt = Console.Read();
                    //ConsoleKeyInfo InputChar = new ConsoleKeyInfo((char)InputCharInt, (ConsoleKey)InputCharInt, false, false, false);
                    if (InputCharInt != -1)
                    {
                        if (InputCharInt == 42) // Note: This will change to 8 once a bug with the PS/2 Controller is fixed
                        {
                            if (!ConsoleMode)
                            {
                                ConsoleA.InterpretKeyInput(new ConsoleKeyInfo('\b', ConsoleKey.Backspace, false, false, false));
                            }
                            else
                            {
                                MainTerminal.BufferBackspace();
                            }
                        }
                        else if (InputCharInt == 47) // Some as the comment above except it'll be 13.
                        {
                            if (!ConsoleMode)
                            {
                                ConsoleA.InterpretKeyInput(new ConsoleKeyInfo('\n', ConsoleKey.Enter, false, false, false));
                            }
                            else
                            {
                                MainTerminal.EnterPressed();
                            }
                        }
                        else
                        {
                            //ConsoleA.AppendText(((char)inputchar).ToString());
                            if (!ConsoleMode)
                            {
                                ConsoleA.InterpretKeyInput(new ConsoleKeyInfo((char)InputCharInt, ConsoleKey.NoName, false, false, false));
                            }
                            else
                            {
                                MainTerminal.WriteToBuffer((char)InputCharInt);
                            }
                        }
                    }

                    if (!ConsoleMode)
                    {
                        //mouseDriver.Draw(canvas);
                        canvas.Display();
                        //driver.Update(0, 0, checked((uint)_SCREEN_WIDTH), checked((uint)_SCREEN_HEIGHT));

                        // Quick and dirty solution but dammit it works.
                        if (!BlackBoxHasBeenFixed && mouseDriver.HasMoved())
                        {
                            canvas.DrawFilledRectangle(BorderPen, new Sys.Graphics.Point(0, 0), 20, 20);
                            BlackBoxHasBeenFixed = true;
                        }
                    }
                }
            } catch
            {

            }
        }

        public static Canvas GetCanvas() { return canvas; }
    }
}
